#define CONFIGURE_INIT
#include "system.h"
#include <signal.h>
#include <errno.h>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <sched.h>
#include "ts.h"
#include "returncode.h"



/*  Critical instant : start time 
    of the application  */
struct timespec critical_instant;



void* periodic_task(void* parameter)
{
        
	struct timespec period=*((struct timespec*)(parameter));
        int number_of_release=0;
	struct timespec nexttime=critical_instant;
	struct timespec waittime;
	struct timespec releasetime;
	struct timespec currenttime;
        int status;

	printf("Thread %x starts : period = %d:%d\n",pthread_self(), period.tv_sec, period.tv_nsec);

        while(number_of_release<3)
        {
               
	releasetime=ts_substract(nexttime, critical_instant);		
        printf("Thread %x is running its %dth release at time %d:%d\n",
	       pthread_self(), number_of_release, releasetime.tv_sec, releasetime.tv_nsec);


	/* Compute the next release time */
  	status=clock_gettime(CLOCK_REALTIME, &currenttime);
    	if(status<0)
  	  returncode("clock_gettime",status);
	nexttime=ts_add(period, nexttime);

        /* wait for the next release time */
	waittime=ts_substract(nexttime, currenttime);
        printf("Thread %x is waiting during %d:%d\n",
	       pthread_self(), waittime.tv_sec, waittime.tv_nsec);
	nanosleep(&waittime, NULL);	
	number_of_release++;

        }

        pthread_exit(NULL);
}




void* POSIX_Init(void *argument)
{

 pthread_attr_t attr;
 pthread_t id;
 struct sched_param param;
 int status;
 
 struct timespec Sensor1;
 struct timespec Sensor2;
 struct timespec Sensor3;
 struct timespec Display1;
 struct timespec Display2;
   
 // Set priorities of the task
 int Prio_Sensor1 =****
 int Prio_Sensor2 =****
 int Prio_Sensor3 =****
 int Prio_Display1=****
 int Prio_Display2=****

 // Set periods of the task
 Sensor1.tv_sec=****   Sensor1.tv_nsec=****
 Sensor2.tv_sec=****   Sensor2.tv_nsec=****
 Sensor3.tv_sec=****   Sensor3.tv_nsec=****
 Display1.tv_sec=****  Display1.tv_nsec=****
 Display2.tv_sec=****  Display2.tv_nsec=****

 
 /* Main thread priority */
 param.sched_priority=100; 
 status=pthread_setschedparam(pthread_self(), SCHED_FIFO, &param);
 if(status<0)
  returncode("pthread_setschedparam",status);


 /* create threads */
 pthread_attr_init(&attr);
 status=pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
 if(status<0)
  returncode("pthread_attr_setinheritsched",status);
 status=pthread_attr_setschedpolicy(&attr,SCHED_FIFO);
 if(status<0)
  returncode("pthread_attr_setschedpolicy",status);


 // Create Sensor1
 param.sched_priority=Prio_Sensor1;
 status=pthread_attr_setschedparam(&attr,&param);
 if(status<0)
  returncode("pthread_attr_setschedparam",status);
 status=pthread_create(&id,&attr,periodic_task,&Sensor1);
 if(status<0)
  returncode("pthread_create",status);

 // Create Sensor2
 param.sched_priority=Prio_Sensor2;
 status=pthread_attr_setschedparam(&attr,&param);
 if(status<0)
  returncode("pthread_attr_setschedparam",status);
 status=pthread_create(&id,&attr,periodic_task,&Sensor2);
 if(status<0)
  returncode("pthread_create",status);
 
  // Create Sensor3
 param.sched_priority=Prio_Sensor3;
 status=pthread_attr_setschedparam(&attr,&param);
 if(status<0)
  returncode("pthread_attr_setschedparam",status);
 status=pthread_create(&id,&attr,periodic_task,&Sensor3);
 if(status<0)
  returncode("pthread_create",status);

 // Create Display1
 param.sched_priority=Prio_Display1;
 status=pthread_attr_setschedparam(&attr,&param);
 if(status<0)
  returncode("pthread_attr_setschedparam",status);
 status=pthread_create(&id,&attr,periodic_task,&Display1);
 if(status<0)
  returncode("pthread_create",status);

 // Create Display2
 param.sched_priority=Prio_Display2;
 status=pthread_attr_setschedparam(&attr,&param);
 if(status<0)
  returncode("pthread_attr_setschedparam",status);
 status=pthread_create(&id,&attr,periodic_task,&Display2);
 if(status<0)
  returncode("pthread_create",status);


 

 /* Measure and store start time of the application */
 status=clock_gettime(CLOCK_REALTIME, &critical_instant);
 if(status<0)
  returncode("clock_gettime",status);

 printf("init.c is completed, start running the periodic tasks\n");
 pthread_exit(NULL);
 return 0; 
}



